function [accuracy, pre_label, pre_value] = EUPAL(labeled_data, labeled_target, unlabeled_data, test_data, test_target, k, alpha, T)

% Syntax
%
%       [accuracy, pre_label, pre_value] = EUPAL(labeled_data, labeled_target, unlabeled_data, test_data, test_target, k, alpha, T)
%
% Description
%
%       EUPAL takes,
%           labeled_data        - A LxD array, the ith instance of labeled instance is stored in labeled_data(i,:)
%           labeled_target      - A LxQ array, if the jth class label is the ground-truth labels for the ith labeled instance, then labeled_target(i,j) equals +1, otherwise labeled_target(i,j) equals 0
%           unlabeled_data      - A UxD array, the ith instance of unlabeled instance is stored in unlabeled_data(i,:)
%           test_data           - A MxD array, the ith instance of testing instance is stored in test_data(i,:)
%           test_target         - A MxQ array, if the jth class label is the ground-truth label for the ith test instance, then test_target(i,j) equals 1; otherwise test_target(i,j) equals 0
%           k                   - The number of nearest neighbors used for kNN aggregation (defalut 5)
%           alpha               - The balancing parameter with 0<alpha<1 (defalut 0.4)
%           T                   - The maximum number of iterations (defalut 50)
%      
%       and returns,
%           accuracy            - Predictive accuracy on the test set
%           pre_label           - A Mx1 array, if the ith test instance is predicted to have the jth class label, then pre_label(i) is j
%           pre_value           - A MxQ array, the numerical output of the ith test instance on the jth class label is stored in Outputs(i,j)

[pl_target] = partial_label_assignment(labeled_data, labeled_target, unlabeled_data, k);
[pl_model, sl_model, mu_pl, mu_sl] = EUPAL_train(labeled_data, labeled_target, unlabeled_data, pl_target, alpha, T);

[accuracy, pre_label, pre_value] = EUPAL_predict(test_data, test_target, pl_model, sl_model, mu_pl, mu_sl);
end