function [accuracy, combine_label, combine_confidence] = EUPAL_predict(test_data, test_target, pl_model, sl_model, mu_pl, mu_sl)

% Syntax
%
%       [accuracy, combine_label, combine_confidence] = EUPAL_predict(test_data, test_target, pl_model, sl_model, mu_pl, mu_sl)
%
% Description
%
%       EUPAL takes,
%           test_data           - A MxD array, the ith instance of testing instance is stored in test_data(i,:)
%           test_target         - A MxQ array, if the jth class label is the ground-truth label for the ith test instance, then test_target(i,j) equals 1; otherwise test_target(i,j) equals 0
%           pl_model            - The partial label training model
%           sl_model            - The supervised training model
%           mu_pl               - The weight of partial label training model
%           mu_sl               - The weight of supervised training model
%      
%       and returns,
%           accuracy            - Predictive accuracy on the test set
%           combine_label       - A Mx1 array, if the ith test instance is predicted to have the jth class label, then pre_label(i) is j
%           combine_confidence  - A MxQ array, the numerical output of the ith test instance on the jth class label is stored in Outputs(i,j)


if nargin<6
    error('Not enough input parameters, please check again.');
end
if size(test_data,1)~=size(test_target,1)
    error('Length of label vector does match the number of instances');
end

test_size = size(test_target,1);

[~, pl_predict_label1, confidence] = IPAL_predict(test_data, test_target', pl_model);
confidence = confidence';
[~,pl_predict_label]=max(pl_predict_label1,[],1);

[~,sl_test_label]=max(test_target,[],2);
[sl_predict_label, ~, dec_val] = libsvmpredict(sl_test_label, test_data, sl_model, '-b 1');

combine_confidence = (confidence+dec_val)/2;
combine_label = pl_predict_label';
for i =1: test_size
    if pl_predict_label(i)~=sl_predict_label(i)
        combine_confidence(i,:) = (confidence(i,:).*mu_pl + dec_val(i,:).*mu_sl)/2;
        [~, combine_label(i)] = max(combine_confidence(i,:), [], 2);
    end
end

accuracy = sum(sl_test_label == combine_label)/test_size;
end
