function [accuarcy,predictLabel,outputValue] = IPAL_predict(test_data,test_target,model)

% This function is the prediction phase of the algorithm. 
%
%    Syntax
%
%  [accuarcy,predictLabel,outputValue] = IPAL_predict(test_data,test_target,model)
% 
%    Description
%
%       IPAL_predict takes,
%           model                       - the model which returned in the disambiguation phase
%           test_data                   - An MxN array, the ith instance of testing instance is stored in test_data(i,:)
%           test_target                 - A QxM array, if the jth class label is the ground-truth label for the ith test instance, then test_target(j,i) equals 1; otherwise test_target(j,i) equals 0
% 
%      and returns,
%            accuarcy                     - Predictive accuracy on the test set
%            predictLabel                 - A QxM array, if the ith test instance is predicted to have the jth class label, then predictLabel(j,i) is 1, otherwise predictLabel(j,i) is 0
%            outputValue                  - A QxM array, the numerical output of the ith test instance on the jth class label is stored in Outputs(j,i)

if nargin<3
    error('Not enough input parameters, please check again.');
end

if model.type~='IPAL'
    error('The input model does not match the prediction model')
end
fea_num = size(test_data,2);
if fea_num ~= model.fea_num
    error('feature size of test data does not match the feature size of training data');
end

if size(test_data,1)~=size(test_target,2)
    error('Length of label vector does match the number of instances');
end

k = model.k;
test_data = normr(test_data);
[testNeighbor,testDist] = knnsearch(model.kdtree,test_data,'k',k);

accNum=0;
y = model.disambiguatedLabel';
data = model.kdtree.X;
[label_num,test_num] = size(test_target);
predictLabel = zeros(label_num,test_num);
outputValue = zeros(label_num,test_num);
for i =1:test_num
    
    
    neighborIns = data(testNeighbor(i,:),:)';
    w = lsqnonneg(neighborIns,test_data(i,:)');
    neighborLabel = y(testNeighbor(i,:),:);
%     w = w(1:k);
    minVal = inf;
    minIdx = -1;
    for label=1:label_num
        
        tmp=neighborLabel(:,label);
        restore = neighborIns*(tmp.*w);
        %                 restore = restore(1:featureNum,:);
        residual = norm(test_data(i,:)'-restore,2);
        outputValue(label,i) = exp(0-residual);
        if residual<minVal
            minVal = residual;
            minIdx = label;
        end
    end
    predictLabel(minIdx,i) = 1;
    if test_target(minIdx,i)==1
        accNum = accNum+1;
    end
    
    outputValue(:, i) = outputValue(:, i)/sum(outputValue(:, i));
    
end
accuarcy = accNum/test_num;
end